﻿using DocumentoDTE.SiiDte;
using GeneGeneradorFacturaLib;
using LiquidTechnologies.Runtime.Net20;
using System;
using System.Collections.Generic;


namespace GeneradorFacturaLib
{
    /// <summary>
    /// Representa la estructura de una Exportación.
    /// </summary>
    public class Exportaciones : IFactura
    {
        /// <summary>
        /// Lista de Detalles de una Exportación.
        /// </summary>
        public List<DetalleB> ListaDetalles { set; get; }

        /// <summary>
        /// Lista de Referencias de una Exportación.
        /// </summary>
        public List<ReferenciaB> ListaReferencias { set; get; }

        /// <summary>
        /// DTE Documento.
        /// </summary>
        private DocumentoDTE.SiiDte.DTEDefType dte;

        /// <summary>
        /// DTE Documento.
        /// </summary>
        public DocumentoDTE.SiiDte.DTEDefType DTE
        {
            get
            {
                return dte;
            }
        }

        /// <summary>
        /// Estructura para las Exportaciones.
        /// </summary>
        public DocumentoDTE.SiiDte.Exportaciones Valor
        {
            get
            {
                return dte.DTE_Choice.Exportaciones;
            }
        }

        /// <summary>
        /// Encabezado/IdDoc de las Exportaciones.
        /// </summary>
        public DocumentoDTE.SiiDte.IdDocB IdDoc
        {
            get
            {
                return dte.DTE_Choice.Exportaciones.Encabezado.IdDoc;
            }
        }

        /// <summary>
        /// Encabezado/Emisor de las Exportaciones.
        /// </summary>
        public DocumentoDTE.SiiDte.EmisorB Emisor
        {
            get
            {
                return dte.DTE_Choice.Exportaciones.Encabezado.Emisor;
            }
        }

        /// <summary>
        /// Encabezado/Receptor de las Exportaciones.
        /// </summary>
        public DocumentoDTE.SiiDte.ReceptorB Receptor
        {
            get
            {
                return dte.DTE_Choice.Exportaciones.Encabezado.Receptor;
            }
        }
        /// <summary>
        /// Encabezado/Totales de las Exportaciones.
        /// </summary>
        public DocumentoDTE.SiiDte.TotalesB Totales
        {
            get
            {
                return dte.DTE_Choice.Exportaciones.Encabezado.Totales;
            }
        }

        /// <summary>
        /// Crea un Objeto Exportaciones.
        /// </summary>
        public Exportaciones()
        {
            ListaDetalles = new List<DetalleB>();
            ListaReferencias = new List<ReferenciaB>();
            
            dte = new DTEDefType();

            dte.DTE_Choice = new DTE_Choice();
            dte.DTE_Choice.Exportaciones = new DocumentoDTE.SiiDte.Exportaciones();

            Valor.Encabezado = new EncabezadoB();
            Valor.Encabezado.IdDoc = new IdDocB();
            Valor.Encabezado.Emisor = new EmisorB();
            Valor.Encabezado.Receptor = new ReceptorB();
            Valor.Encabezado.Totales = new TotalesB();
        }

        /// <summary>
        /// Establece los valores para Encabezado/IdDoc.
        /// </summary>
        /// <param name="TipoDTE">Tipo de DTE</param>
        /// <param name="Folio">Folio del documento</param>
        /// <param name="FchEmis">Fecha de emisión</param>
        public void SetEncabezadoIdDoc(
            string TipoDTE,
            long Folio,
            DateTime FchEmis)
        {
            IdDoc.Folio = new BigInteger(Folio);
            IdDoc.FchEmis = new LiquidTechnologies.Runtime.Net20.XmlDateTime(FchEmis);
            IdDoc.TipoDTE = DocumentoDTE.SiiDte.Enumerations.EXPTypeFromString(TipoDTE);
        }

        /// <summary>
        /// Establece los valores para Encabezado/Emisor.
        /// </summary>
        /// <param name="RUTEmisor">RUT del emisor</param>
        /// <param name="RznSoc">Razón social del emisor</param>
        /// <param name="GiroEmis">Giro del emisor</param>
        /// <param name="Actectos">Actectos del emisor</param>
        /// <param name="DirOrigen">Dirección origen del emisor</param>
        /// <param name="CmnaOrigen">Comuna origen del emisor</param>
        /// <param name="CiudadOrigen">Ciudad origen del emisor</param>
        public void SetEncabezadoEmisor(
            string RUTEmisor,
            string RznSoc,
            string GiroEmis,
            string Actectos,
            string DirOrigen,
            string CmnaOrigen,
            string CiudadOrigen
            )
        {
            Emisor.RUTEmisor = RUTEmisor;
            Emisor.RznSoc = RznSoc;
            Emisor.GiroEmis = GiroEmis;
            Emisor.DirOrigen = DirOrigen;

            Emisor.Acteco.Clear();
            foreach (var item in Actectos.Split(','))
            {
                long ateco = 0;
                long.TryParse(item, out ateco);
                Emisor.Acteco.Add(new BigInteger(ateco));
            }
            Emisor.CmnaOrigen = CmnaOrigen;
            Emisor.CiudadOrigen = CiudadOrigen;
        }

        /// <summary>
        /// Establece los valores para Encabezado/Receptor.
        /// </summary>
        /// <param name="RUTRecep">RUT del Receptor</param>
        /// <param name="RznSocRecep">Razón social del Receptor</param>
        /// <param name="GiroRecep">Giro del Receptor</param>
        /// <param name="DirRecep">Dirección del Receptor</param>
        /// <param name="CmnaRecep">Comuna del Receptor</param>
        /// <param name="CiudadRecep">Ciudad del Receptor</param>
        public void SetEncabezadoReceptor(
            string RUTRecep,
            string RznSocRecep,
            string GiroRecep,
            string DirRecep,
            string CmnaRecep,
            string CiudadRecep)
        {
            Receptor.RUTRecep = RUTRecep;
            Receptor.RznSocRecep = RznSocRecep;
            Receptor.GiroRecep = GiroRecep;
            Receptor.DirRecep = DirRecep;
            Receptor.CmnaRecep = CmnaRecep;
            Receptor.CiudadRecep = CiudadRecep;
        }

        /// <summary>
        /// Establece los valores para Encabezado/Totales.
        /// </summary>
        /// <param name="MntTotal">Monto total</param>
        /// <param name="MntExe">Monto exento</param>
        public void SetEncabezadoTotales(
            decimal MntTotal,
            decimal MntExe)
        {
            Totales.MntTotal = MntTotal;
            Totales.MntExe = MntExe;
        }

        #region Detalle

        /// <summary>
        /// Adiciona un item a los detalles del documento.
        /// </summary>
        /// <param name="NmbItem">Nombre del item</param>
        /// <param name="MontoItem">Monto del item</param>
        /// <param name="IndExe">Indicador exento</param>
        /// <param name="DscItem">Descripción del item</param>
        /// <param name="QtyItem">Cantidad del item</param>
        /// <param name="UnmdItem">Unidad de medida del item</param>
        /// <param name="PrcItem">Precio del item</param>
        /// <param name="ListaCdgItem">Lista de códigos del item</param>
        public void AdicionarDetalle(
            string NmbItem,
            decimal MontoItem,
            string IndExe,
            string DscItem,
            decimal QtyItem,
            string UnmdItem,
            decimal PrcItem,
            IList<CdgItemB> ListaCdgItem)
        {
            ActualizarDetalleEnIndice(
            NmbItem,
            MontoItem,
            IndExe,
            DscItem,
            QtyItem,
            UnmdItem,
            PrcItem,
            ListaCdgItem,
                -1);
        }

        /// <summary>
        /// Actualiza un item de los detalles del documento dada la posición del mismo.
        /// </summary>
        /// <param name="NmbItem">Nombre del item</param>
        /// <param name="MontoItem">Monto del item</param>
        /// <param name="IndExe">Indicador exento</param>
        /// <param name="DscItem">Descripción del item</param>
        /// <param name="QtyItem">Cantidad del item</param>
        /// <param name="UnmdItem">Unidad de medida del item</param>
        /// <param name="PrcItem">Precio del item</param>
        /// <param name="CdgItemList">Lista de códigos del item</param>
        /// <param name="indice">indice del item a actualizar</param>
        public void ActualizarDetalleEnIndice(
            string NmbItem,
            decimal MontoItem,
            string IndExe,
            string DscItem,
            decimal QtyItem,
            string UnmdItem,
            decimal PrcItem,
            IList<CdgItemB> CdgItemList,
            int indice)
        {
            DocumentoDTE.SiiDte.DetalleB detL = new DocumentoDTE.SiiDte.DetalleB();

            detL.NroLinDet = new BigInteger(ListaDetalles.Count + 1);

            detL.NmbItem = NmbItem;
            detL.MontoItem = MontoItem;
            
            if (IndExe != null)
                detL.IndExe = Enumerations.DetalleB_IndExeFromString(IndExe);

            detL.DscItem = DscItem;
            detL.QtyItem = QtyItem;
            detL.UnmdItem = UnmdItem;
            detL.PrcItem = PrcItem;

            detL.CdgItem.Clear();
            foreach (var cdgItemL in CdgItemList)
            {
                detL.CdgItem.Add(cdgItemL);
            }

            if (indice > -1)
            {
                ListaDetalles[indice] = detL;
            }
            else
            {
                ListaDetalles.Add(detL);
            }
        }

        /// <summary>
        /// Elimina un item de la lista de detalles.
        /// </summary>
        /// <param name="indice">indice del item a eliminar</param>
        public void EliminarDetalleEnIndice(int indice)
        {
            ListaDetalles.RemoveAt(indice);
        }

        /// <summary>
        /// Crea un objeto CdgItemB.
        /// </summary>
        /// <param name="TpoCodigo">Tipo de código</param>
        /// <param name="VlrCodigo">Valor del código</param>
        /// <returns>Objeto de tipo CdgItemB</returns>
        public CdgItemB CrearDetalleCdgItem(
            string TpoCodigo,
            string VlrCodigo)
        {
            DocumentoDTE.SiiDte.CdgItemB cdgItemL = new DocumentoDTE.SiiDte.CdgItemB();
            cdgItemL.TpoCodigo = TpoCodigo;
            cdgItemL.VlrCodigo = VlrCodigo;
            return cdgItemL;
        }

        #endregion

        #region Referencia

        /// <summary>
        /// Adiciona una Referencia a la lista de Referencias.
        /// </summary>
        /// <param name="TpoDocRef">Tipo de documento de la Referencia</param>
        /// <param name="FolioRef">Folio de la Referencia</param>
        /// <param name="FchRef">Fecha de la Referencia</param>
        /// <param name="RazonRef">Razón de la Referencia</param>
        public void AdicionarReferencia(
            string TpoDocRef,
            string FolioRef,
            DateTime FchRef,
            string RazonRef)
        {
            ActualizarReferenciaEnIndice(TpoDocRef, FolioRef, FchRef, RazonRef, -1);
        }

        /// <summary>
        /// Actualiza una Referencia a la lista de Referencias dado el indice de la misma.
        /// </summary>
        /// <param name="TpoDocRef">Tipo de documento de la Referencia</param>
        /// <param name="FolioRef">Folio de la Referencia</param>
        /// <param name="FchRef">Fecha de la Referencia</param>
        /// <param name="RazonRef">Razón de la Referencia</param>
        /// <param name="indice"></param>
        public void ActualizarReferenciaEnIndice(
            string TpoDocRef,
            string FolioRef,
            DateTime FchRef,
            string RazonRef,
            int indice)
        {
            DocumentoDTE.SiiDte.ReferenciaB referenciaL = new DocumentoDTE.SiiDte.ReferenciaB();

            referenciaL.NroLinRef = new BigInteger(ListaReferencias.Count + 1);

            referenciaL.TpoDocRef = TpoDocRef;
            referenciaL.FolioRef = FolioRef;
            referenciaL.FchRef = new LiquidTechnologies.Runtime.Net20.XmlDateTime(FchRef);
            if (!String.IsNullOrEmpty(RazonRef))
                referenciaL.RazonRef = RazonRef;

            if (indice > -1)
            {
                ListaReferencias[indice] = referenciaL;
            }
            else
            {
                ListaReferencias.Add(referenciaL);
            }
        }

        /// <summary>
        /// Elimina una Referencia de la lista de Referencias dado el indice de la misma.
        /// </summary>
        /// <param name="indice">indice de la Referencia en la lista</param>
        public void EliminarReferenciaEnIndice(int indice)
        {
            ListaReferencias.RemoveAt(indice);
        }

        #endregion

        /// <summary>
        /// Obtiene el XML representativo de la Exportación.
        /// </summary>
        /// <returns>string con el XML de la Exportación</returns>
        public string ObtenerXml()
        {
            #region Listas

            var indice = 0;

            //Liquidacion/Detalle
            Valor.Detalle.Clear();
            foreach (var detL in ListaDetalles)
            {
                detL.NroLinDet = new BigInteger(++indice);
                Valor.Detalle.Add(detL);
            }

            //Liquidacion/Referencia
            Valor.Referencia.Clear();
            indice = 0;
            foreach (var referenciaL in ListaReferencias)
            {
                referenciaL.NroLinRef = new BigInteger(++indice);
                Valor.Referencia.Add(referenciaL);
            }

            #endregion

            return GeneradorXmlDTE.ObtenerXML(DTE);
        }

    }
}
