﻿using System;


namespace GeneradorFacturaLib
{
    /// <summary>
    /// Clase utilitaria para realizar algunas operaciones con los campos de las Facturas.
    /// </summary>
    internal static class Extension
    {
        /// <summary>
        /// Método genérico que obtiene el Enummerado dado un objeto.
        /// </summary>
        /// <typeparam name="T">Tipo del Enumerado</typeparam>
        /// <param name="valor">objeto</param>
        /// <returns>Objeto de tipo Enum</returns>
        public static T EnumObj<T>(object valor)
        {
            return EnumObj<T>(valor.ToString());
        }

        /// <summary>
        /// Método genérico que obtiene el Enummerado dado un valor string.
        /// </summary>
        /// <typeparam name="T">Tipo del Enumerado</typeparam>
        /// <param name="valor">valor string que representa el enumerado</param>
        /// <returns>Objeto de tipo Enum</returns>
        public static T EnumObj<T>(string valor)
        {
            return (T)Enum.Parse(typeof(T), valor);
        }

        /// <summary>
        ///  Método que obtiene el Enummerado dado un valor string y el tipo de Enumerado.
        /// </summary>
        /// <param name="tipo">Tipo del Enummerado</param>
        /// <param name="valor">valor string que representa el enumerado</param>
        /// <returns>Objeto de tipo Enum</returns>
        public static object EnumObj(Type tipo, string valor)
        {
            return Enum.Parse(tipo, valor);
        }

        /// <summary>
        ///  Método que obtiene el Enummerado dado un objeto y el tipo de Enumerado.
        /// </summary>
        /// <param name="tipo">Tipo del Enummerado</param>
        /// <param name="valor">objeto que representa el enumerado</param>
        /// <returns>Objeto de tipo Enum</returns>
        public static object EnumObj(Type tipo, object valor)
        {
            return EnumObj(tipo, valor.ToString());
        }

        /*/// <summary>
        /// Establece el valor de una propiedad de las Facturas el cual es opcional.
        /// </summary>
        /// <param name="obj">Objeto de tipo IFactura</param>
        /// <param name="nombrePropiedad">Nombre de la propiedad</param>
        /// <param name="tieneValor">Indica si tiene valor</param>
        /// <param name="valor">Valor a setear</param>
        public static void SetValorOpcional(this object obj, string nombrePropiedad, bool tieneValor, object valor)
        {
            if (!tieneValor)
            {
                var esValidaNombrePropiedad = string.Format("IsValid{0}", nombrePropiedad);
                var esValidaPropiedad = obj.GetType().GetProperty(esValidaNombrePropiedad);
                if (null != esValidaPropiedad)
                {
                    try
                    {
                        esValidaPropiedad.SetValue(obj, false, null);
                    }
                    catch { }
                }
                return;
            }

            var propiedad = obj.GetType().GetProperty(nombrePropiedad);
            if (null == propiedad) return;

            if (propiedad.PropertyType.IsEnum)
            {
                object valorEnum = Extension.EnumObj(propiedad.PropertyType, valor.ToString());
                propiedad.SetValue(obj, valorEnum, null);
            }
            else
            {
                var tipoValor = valor.GetType();
                if (!propiedad.PropertyType.IsAssignableFrom(tipoValor))
                {
                    if (propiedad.PropertyType == typeof(string))
                    {
                        valor = valor.ToString();
                    }
                    else
                    {
                        var valorAnt = valor;
                        valor = Activator.CreateInstance(propiedad.PropertyType, new object[] { valor });
                        if (null == valor)
                        {
                            throw new Exception(string.Format("Can not convert value {0} to property type {1}.", valorAnt, propiedad.PropertyType.FullName));
                        }
                    }
                }
                propiedad.SetValue(obj, valor, null);
            }
        }*/
    }
}
