﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Web;
using System.Xml;

namespace GeneradorFacturaLib
{
    /// <summary>
    /// Clase utilitaria para generar el XML de los documentos de tipo Boleta.
    /// </summary>
    public class GeneradorXmlBoleta
    {
        /// <summary>
        /// Obtiene el XML de una boleta.
        /// </summary>
        /// <param name="boleta">Boleta a la cual se le va a obtener el XML</param>
        /// <returns>string que representa el XML de la Boleta</returns>
        public static string ObtenerXML(DocumentoBoleta.SiiDte.BOLETADefType boleta)
        {

            #region Llenar Campos Boleta

            boleta.Version = 1;

            LlenarDocumento(boleta);

            #endregion

            var xsc = new LiquidTechnologies.Runtime.Net20.XmlSerializationContext();
            xsc.NamespaceAliases.Add("ABCXYZ", boleta.Namespace);

            var stream = new MemoryStream();
            boleta.ToXmlStream(stream, true, Formatting.Indented, Encoding.GetEncoding("ISO-8859-1"), LiquidTechnologies.Runtime.Net20.EOLType.CRLF, xsc);

            var xmlBytes = stream.ToArray();

            var xml = Encoding.UTF8.GetString(xmlBytes);
            var xmlStr = xml
                .Replace(":ABCXYZ", "")
                .Replace("ABCXYZ:", "")
                .Replace("<!--Created by Liquid XML Data Binding Libraries (www.liquid-technologies.com) for GDE S.A. (1 * Developer Bundle)-->\r\n", "")
                .Replace("<!--Created by Liquid XML Data Binding Libraries (www.liquid-technologies.com) for SOUTH CONSULTING SIGNATURE S.A. (1 * Developer Edition)-->\r\n", "")
                .Replace("xmlns=\"http://www.sii.cl/SiiDte\" ", "")
                .Replace("BOLETADefType", "DTE");

            return xmlStr;
        }

        /// <summary>
        /// Llena la estructura TED de un documento Boleta.
        /// </summary>
        /// <param name="boleta">Boleta a la cual se le va a llenar la estructura TED</param>
        private static void LlenarDocumento(DocumentoBoleta.SiiDte.BOLETADefType boleta)
        {
            var doc = boleta.Documento;

            doc.ID = "ID_1";

            //TED
            doc.TED = new DocumentoBoleta.SiiDte.TED();

            doc.TED.Version = "1.0";

            //TED/DD
            doc.TED.DD = new DocumentoBoleta.SiiDte.DD();

            doc.TED.DD.RE = doc.Encabezado.Receptor.RUTRecep;
            doc.TED.DD.TD = doc.Encabezado.IdDoc.TipoDTE;
            doc.TED.DD.F = doc.Encabezado.IdDoc.Folio;
            doc.TED.DD.FE = doc.Encabezado.IdDoc.FchEmis;
            doc.TED.DD.RR = doc.Encabezado.Receptor.RUTRecep;
            doc.TED.DD.RSR = "RSR";
            doc.TED.DD.MNT = (ulong)doc.Encabezado.Totales.MntTotal.LongValue();
            doc.TED.DD.IT1 = "IT1";

            //TED/Fake CAF
            doc.TED.DD.CAF = new DocumentoBoleta.SiiDte.CAF();
            doc.TED.DD.CAF.DA = new DocumentoBoleta.SiiDte.DA();

            doc.TED.DD.CAF.DA.RE = "00000000-K";
            doc.TED.DD.CAF.DA.RS = "A";
            doc.TED.DD.CAF.DA.TD = DocumentoBoleta.SiiDte.Enumerations.DTEType.n39;
            doc.TED.DD.CAF.DA.FA = new LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1);
            doc.TED.DD.CAF.DA.IDK = 0;

            doc.TED.DD.CAF.DA.DA_Choice = new DocumentoBoleta.SiiDte.DA_Choice();
            doc.TED.DD.CAF.DA.DA_Choice.RSAPK = new DocumentoBoleta.SiiDte.RSAPK();
            doc.TED.DD.CAF.DA.DA_Choice.RSAPK.M = new LiquidTechnologies.Runtime.Net20.BinaryData(new byte[] { });
            doc.TED.DD.CAF.DA.DA_Choice.RSAPK.E = new LiquidTechnologies.Runtime.Net20.BinaryData(new byte[] { });

            doc.TED.DD.CAF.DA.RNG = new DocumentoBoleta.SiiDte.RNG();
            doc.TED.DD.CAF.DA.RNG.D = 1;
            doc.TED.DD.CAF.DA.RNG.H = 1;

            doc.TED.DD.CAF.FRMA = new DocumentoBoleta.SiiDte.FRMA();
            doc.TED.DD.CAF.FRMA.Algoritmo = "SHA1withRSA";

            doc.TED.DD.TSTED = new LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1, 0, 0, 0);

            doc.TED.FRMT = new DocumentoBoleta.SiiDte.FRMT();
            doc.TED.FRMT.Algoritmo = DocumentoBoleta.EnvioBOLETA_v11Lib.Enumerations.FRMT_Algoritmo.SHA1withRSA;

            doc.TmstFirma = new LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1, 0, 0, 0);
        }
    }
}
