﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Web;
using System.Xml;

namespace GeneradorFacturaLib
{
    /// <summary>
    /// Clase utilitaria para generar el XML de los documentos DTE.
    /// </summary>
    public class GeneradorXmlDTE
    {
        /// <summary>
        /// Obtiene el XML de un DTE.
        /// </summary>
        /// <param name="boleta">DTE al cual se le va a obtener el XML</param>
        /// <returns>string que representa el XML del DTE</returns>
        public static string ObtenerXML(DocumentoDTE.SiiDte.DTEDefType dte)
        {

            #region Llenar campos DTE

            dte.Version = 1;

            if (dte.DTE_Choice.Documento != null)
            {
                LlenarDocumento(dte.DTE_Choice.Documento as DocumentoDTE.SiiDte.Documento);
            }
            else if (dte.DTE_Choice.Exportaciones != null)
            {
                LlenarExportaciones(dte.DTE_Choice.Exportaciones as DocumentoDTE.SiiDte.Exportaciones);
            }
            else if (dte.DTE_Choice.Liquidacion != null)
            {
                LlenarLiquidacion(dte.DTE_Choice.Liquidacion as DocumentoDTE.SiiDte.Liquidacion);
            }

            #endregion

            var xsc = new LiquidTechnologies.Runtime.Net20.XmlSerializationContext();
            xsc.NamespaceAliases.Add("ABCXYZ", dte.Namespace);

            var stream = new MemoryStream();
            dte.ToXmlStream(stream, true, Formatting.Indented, Encoding.GetEncoding("ISO-8859-1"), LiquidTechnologies.Runtime.Net20.EOLType.CRLF, xsc);

            var xmlBytes = stream.ToArray();

            var xml = Encoding.UTF8.GetString(xmlBytes);
            var xmlStr = xml
                .Replace(":ABCXYZ", "")
                .Replace("ABCXYZ:", "")
                .Replace("<!--Created by Liquid XML Data Binding Libraries (www.liquid-technologies.com) for GDE S.A. (1 * Developer Bundle)-->\r\n", "")
                .Replace("<!--Created by Liquid XML Data Binding Libraries (www.liquid-technologies.com) for SOUTH CONSULTING SIGNATURE S.A. (1 * Developer Edition)-->\r\n", "")
                .Replace("xmlns=\"http://www.sii.cl/SiiDte\" ", "")
                .Replace("DTEDefType", "DTE");

            return xmlStr;
        }

        /// <summary>
        /// Llena la estructura TED de un DTE de tipo Documento.
        /// </summary>
        /// <param name="doc">Documento al cual se le va a llenar la estructura TED</param>
        private static void LlenarDocumento(DocumentoDTE.SiiDte.Documento doc)
        {
            doc.ID = "ID_1";

            //TED
            doc.TED = new DocumentoDTE.SiiDte.TED();

            doc.TED.Version = "1.0";

            //TED/DD
            doc.TED.DD = new DocumentoDTE.SiiDte.DD();

            doc.TED.DD.RE = doc.Encabezado.Emisor.RUTEmisor;
            doc.TED.DD.TD = doc.Encabezado.IdDoc.TipoDTE;
            doc.TED.DD.F = doc.Encabezado.IdDoc.Folio;

            doc.TED.DD.FE = doc.Encabezado.IdDoc.FchEmis;
            doc.TED.DD.RR = doc.Encabezado.Receptor.RUTRecep;
            doc.TED.DD.RSR = "RSR";
            doc.TED.DD.MNT = (ulong)doc.Encabezado.Totales.MntTotal.LongValue();
            doc.TED.DD.IT1 = "IT1";

            //TED/Fake CAF
            doc.TED.DD.CAF = new DocumentoDTE.SiiDte.CAF();
            doc.TED.DD.CAF.DA = new DocumentoDTE.SiiDte.DA();

            doc.TED.DD.CAF.DA.RE = "00000000-K";
            doc.TED.DD.CAF.DA.RS = "A";
            doc.TED.DD.CAF.DA.TD = DocumentoDTE.SiiDte.Enumerations.DTEType.n33;
            doc.TED.DD.CAF.DA.FA = new LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1);
            doc.TED.DD.CAF.DA.IDK = 0;

            doc.TED.DD.CAF.DA.DA_Choice = new DocumentoDTE.SiiDte.DA_Choice();
            doc.TED.DD.CAF.DA.DA_Choice.RSAPK = new DocumentoDTE.SiiDte.RSAPK();
            doc.TED.DD.CAF.DA.DA_Choice.RSAPK.M = new LiquidTechnologies.Runtime.Net20.BinaryData(new byte[] { });
            doc.TED.DD.CAF.DA.DA_Choice.RSAPK.E = new LiquidTechnologies.Runtime.Net20.BinaryData(new byte[] { });

            doc.TED.DD.CAF.DA.RNG = new DocumentoDTE.SiiDte.RNG();
            doc.TED.DD.CAF.DA.RNG.D = 1;
            doc.TED.DD.CAF.DA.RNG.H = 1;

            doc.TED.DD.CAF.FRMA = new DocumentoDTE.SiiDte.FRMA();
            doc.TED.DD.CAF.FRMA.Algoritmo = "SHA1withRSA";

            doc.TED.DD.TSTED = new LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1, 0, 0, 0);

            doc.TED.FRMT = new DocumentoDTE.SiiDte.FRMT();
            doc.TED.FRMT.Algoritmo = DocumentoDTE.EnvioDTE_v10Lib.Enumerations.FRMT_Algoritmo.SHA1withRSA;

            doc.TmstFirma = new LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1, 0, 0, 0);
        }

        /// <summary>
        /// Llena la estructura TED de un DTE de tipo Exportaciones.
        /// </summary>
        /// <param name="doc">Exportación a la cual se le va a llenar la estructura TED</param>
        private static void LlenarExportaciones(DocumentoDTE.SiiDte.Exportaciones doc)
        {
            doc.ID = "ID_1";

            //TED
            doc.TED = new DocumentoDTE.SiiDte.TEDB();

            doc.TED.Version = "1.0";

            //TED/DD
            doc.TED.DD = new DocumentoDTE.SiiDte.DDB();

            doc.TED.DD.RE = doc.Encabezado.Emisor.RUTEmisor;
            doc.TED.DD.TD = doc.Encabezado.IdDoc.TipoDTE;
            doc.TED.DD.F = doc.Encabezado.IdDoc.Folio;
            doc.TED.DD.FE = doc.Encabezado.IdDoc.FchEmis;
            doc.TED.DD.RR = doc.Encabezado.Receptor.RUTRecep;
            doc.TED.DD.RSR = "RSR";
            doc.TED.DD.MNT = doc.Encabezado.Totales.MntTotal;
            doc.TED.DD.IT1 = "IT1";

            //TED/Fake CAF
            doc.TED.DD.CAF = new DocumentoDTE.SiiDte.CAFB();
            doc.TED.DD.CAF.DA = new DocumentoDTE.SiiDte.DAB();

            doc.TED.DD.CAF.DA.RE = "00000000-K";
            doc.TED.DD.CAF.DA.RS = "A";
            doc.TED.DD.CAF.DA.TD = DocumentoDTE.SiiDte.Enumerations.EXPType.n110;
            doc.TED.DD.CAF.DA.FA = new LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1);
            doc.TED.DD.CAF.DA.IDK = 0;

            doc.TED.DD.CAF.DA.DA1_Choice = new DocumentoDTE.SiiDte.DA1_Choice();
            doc.TED.DD.CAF.DA.DA1_Choice.RSAPK = new DocumentoDTE.SiiDte.RSAPKB();
            doc.TED.DD.CAF.DA.DA1_Choice.RSAPK.M = new LiquidTechnologies.Runtime.Net20.BinaryData(new byte[] { });
            doc.TED.DD.CAF.DA.DA1_Choice.RSAPK.E = new LiquidTechnologies.Runtime.Net20.BinaryData(new byte[] { });

            doc.TED.DD.CAF.DA.RNG = new DocumentoDTE.SiiDte.RNGB();
            doc.TED.DD.CAF.DA.RNG.D = 1;
            doc.TED.DD.CAF.DA.RNG.H = 1;

            doc.TED.DD.CAF.FRMA = new DocumentoDTE.SiiDte.FRMAB();
            doc.TED.DD.CAF.FRMA.Algoritmo = "SHA1withRSA";

            doc.TED.DD.TSTED = new LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1, 0, 0, 0);

            doc.TED.FRMT = new DocumentoDTE.SiiDte.FRMTB();
            doc.TED.FRMT.Algoritmo = DocumentoDTE.EnvioDTE_v10Lib.Enumerations.FRMTB_Algoritmo.SHA1withRSA;

            doc.TmstFirma = new LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1, 0, 0, 0);
        }

        /// <summary>
        /// Llena la estructura TED de un DTE de tipo Liquidación.
        /// </summary>
        /// <param name="doc">Liquidación a la cual se le va a llenar la estructura TED</param>
        private static void LlenarLiquidacion(DocumentoDTE.SiiDte.Liquidacion doc)
        {
            doc.ID = "ID_1";

            //TED
            doc.TED = new DocumentoDTE.SiiDte.TEDA();

            doc.TED.Version = "1.0";

            //TED/DD
            doc.TED.DD = new DocumentoDTE.SiiDte.DDA();

            doc.TED.DD.RE = doc.Encabezado.Receptor.RUTRecep;
            doc.TED.DD.TD = doc.Encabezado.IdDoc.TipoDTE;
            doc.TED.DD.F = doc.Encabezado.IdDoc.Folio;
            doc.TED.DD.FE = doc.Encabezado.IdDoc.FchEmis;
            doc.TED.DD.RR = doc.Encabezado.Receptor.RUTRecep;
            doc.TED.DD.RSR = "RSR";
            doc.TED.DD.MNT = (ulong)doc.Encabezado.Totales.MntTotal.LongValue();
            doc.TED.DD.IT1 = "IT1";

            //TED/Fake CAF
            doc.TED.DD.CAF = new DocumentoDTE.SiiDte.CAFA();
            doc.TED.DD.CAF.DA = new DocumentoDTE.SiiDte.DAA();

            doc.TED.DD.CAF.DA.RE = "00000000-K";
            doc.TED.DD.CAF.DA.RS = "A";
            doc.TED.DD.CAF.DA.TD = DocumentoDTE.SiiDte.Enumerations.LIQType.n43;
            doc.TED.DD.CAF.DA.FA = new LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1);
            doc.TED.DD.CAF.DA.IDK = 0;

            doc.TED.DD.CAF.DA.DA0_Choice = new DocumentoDTE.SiiDte.DA0_Choice();
            doc.TED.DD.CAF.DA.DA0_Choice.RSAPK = new DocumentoDTE.SiiDte.RSAPKA();
            doc.TED.DD.CAF.DA.DA0_Choice.RSAPK.M = new LiquidTechnologies.Runtime.Net20.BinaryData(new byte[] { });
            doc.TED.DD.CAF.DA.DA0_Choice.RSAPK.E = new LiquidTechnologies.Runtime.Net20.BinaryData(new byte[] { });

            doc.TED.DD.CAF.DA.RNG = new DocumentoDTE.SiiDte.RNGA();
            doc.TED.DD.CAF.DA.RNG.D = 1;
            doc.TED.DD.CAF.DA.RNG.H = 1;

            doc.TED.DD.CAF.FRMA = new DocumentoDTE.SiiDte.FRMAA();
            doc.TED.DD.CAF.FRMA.Algoritmo = "SHA1withRSA";

            doc.TED.DD.TSTED = new LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1, 0, 0, 0);

            doc.TED.FRMT = new DocumentoDTE.SiiDte.FRMTA();
            doc.TED.FRMT.Algoritmo = DocumentoDTE.EnvioDTE_v10Lib.Enumerations.FRMTA_Algoritmo.SHA1withRSA;

            doc.TmstFirma = new LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1, 0, 0, 0);
        }

    }
}
