﻿using System;


namespace GeneradorFacturaLib
{
    /// <summary>
    /// Clase utilitaria para realizar validaciones a los campos de tipo RUT.
    /// </summary>
    public static class ValidadorRUT
    {

        /// <summary>
        /// Determina cuando el RUT es válido o no de acuerdo a http://es.wikipedia.org/wiki/Rol_%C3%9Anico_Tributario
        /// </summary>
        /// <param name="rut">Rut a validar</param>
        /// <returns></returns>
        public static bool EsValido(string rut)
        {
            if (string.IsNullOrEmpty(rut))
            {
                return false;
            }
            var partesRut = rut.ToString().Split(new string[] { "-" }, StringSplitOptions.RemoveEmptyEntries);
            if (partesRut.Length != 2)
            {
                return false;
            }
            if (partesRut[0].Length > 8)
            {
                return false;
            }
            int intRutPart;
            if (int.TryParse(partesRut[0], out intRutPart))
            {
                var verificarDigito = VerificarDigito(intRutPart);
                if (verificarDigito == partesRut[1])
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// Verifica la cantidad de dígitos de las partes del RUT
        /// </summary>
        /// <param name="rut">cantidad de dígitos</param>
        /// <returns></returns>
        public static string VerificarDigito(int rut)
        {
            int digit;
            int counter;
            int multiple;
            int sumatory;
            string rutDigit;

            counter = 2;
            sumatory = 0;

            while (rut != 0)
            {
                multiple = (rut % 10) * counter;
                sumatory += multiple;
                rut = rut / 10;
                counter = counter + 1;
                if (counter == 8)
                {
                    counter = 2;
                }
            }

            digit = 11 - (sumatory % 11);
            rutDigit = digit.ToString().Trim();
            if (digit == 10)
            {
                rutDigit = "K";
            }
            if (digit == 11)
            {
                rutDigit = "0";
            }

            return rutDigit;
        }

    }
}
