﻿using System.IO;
using DocumentoBoleta.SiiDte;
using LiquidTechnologies.Runtime.Net20;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using System.Xml;
using System.Collections;
using System.Collections.Specialized;
using GenerarFactura;
using GeneGeneradorFacturaLib;
using GeneradorFacturaLib;

namespace GenerarFactura
{
    /// <summary>
    /// Formulario para gestionar los documentos de tipo Boleta.
    /// </summary>
    public partial class BolForm : Form
    {
        /// <summary>
        /// Lista para almacenar los controles de entrada.
        /// </summary>
        private List<Control> controlesEntrada;

        /// <summary>
        /// Lista para almacenar los códigos de la sección de detalles.
        /// </summary>
        public List<CdgItem> ListaCdgItem { set; get; }

        /// <summary>
        /// Definición de las diferentes reglas de validación para los campos de la interfaz de usuario en este formulario.
        /// </summary>
        public Dictionary<string, Validacion> definicionesValidacion = new Dictionary<string, Validacion>
        {
            {"textRUTEmisor", new Validacion {LabelCampo = "RUT del Emisor", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true}, {"rut", true} }}},
            {"textRznSoc", new Validacion {LabelCampo = "Razón social del Emisor", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textGiroEmis", new Validacion {LabelCampo = "Giro del Emisor", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textDirOrigen", new Validacion {LabelCampo = "Dirección del Emisor", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textCmnaOrigen", new Validacion {LabelCampo = "Comuna del Emisor", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textCiudadOrigen", new Validacion {LabelCampo = "Ciudad del Emisor", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textRUTRecep", new Validacion {LabelCampo = "RUT del Cliente", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true}, {"rut", true} }}},
            {"textRznSocRecep", new Validacion {LabelCampo = "Razón social del Cliente", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textDirRecep", new Validacion {LabelCampo = "Dirección del Cliente", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textCmnaRecep", new Validacion {LabelCampo = "Comuna del Cliente", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textCiudadRecep", new Validacion {LabelCampo = "Ciudad del Cliente", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},

            {"textFolio", new Validacion {LabelCampo = "Folio", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true}, {"num", true} }}},
            {"textMntNeto", new Validacion {LabelCampo = "Monto Neto", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"num", true} }}},
            {"textIva", new Validacion {LabelCampo = "IVA", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"num", true} }}},
            {"textMntExe", new Validacion {LabelCampo = "Monto Exento", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"num", true} }}},
            {"textMntTotal", new Validacion {LabelCampo = "Monto Total", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true}, {"num", true} }}},
            {"textIpDteBox", new Validacion {LabelCampo = "IP DTEBOX", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textLlaveAutenticacion", new Validacion {LabelCampo = "Llave de Autenticación", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},

            {"textNmbItem", new Validacion {LabelCampo = "Nombre", Seccion = "Detalle", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textQtyItem", new Validacion {LabelCampo = "Cantidad", Seccion = "Detalle", Reglas = new Dictionary<string, object> { {"req", true}, {"num", true}, {"min", new decimal(0.000001)} }}},
            {"textPrcItem", new Validacion {LabelCampo = "Precio U.", Seccion = "Detalle", Reglas = new Dictionary<string, object> { {"req", true}, {"num", true}, {"min", new decimal(0.000001)} }}},
            {"textMontoItem", new Validacion {LabelCampo = "Monto", Seccion = "Detalle", Reglas = new Dictionary<string, object> { {"req", true}, {"num", true} }}},

            {"textCodRef", new Validacion {LabelCampo = "Código", Seccion = "Referencia", Reglas = new Dictionary<string, object> { {"req", true} }}},

            {"dataGridDetalles", new Validacion {LabelCampo = "Detalle", Seccion = "Detalles", Reglas = new Dictionary<string, object> { {"req", true} }}},
            
        };

        /// <summary>
        /// Documento de tipo Boleta a gestionar.
        /// </summary>
        public GeneradorFacturaLib.Boleta Documento { set; get; }

        public BolForm()
        {
            InitializeComponent();

            ListaCdgItem = new List<CdgItem>();

            UtilIU.LlenarValoresDesde<Enumerations.Detalle_IndExe>(comboIndExe);

            var doc = new List<string> { "35", "38", "39", "41", "5001", "5002" };
            var listTipoDoc = AsistenteEnum.ListaTipoDocumento().FindAll(i => doc.Contains(i.Key));

            UtilIU.LlenarValoresDesdeLista(comboTipoDTE, listTipoDoc);

            var listUM = AsistenteEnum.ListaUnidadMedida();
            listUM.Insert(0, new KeyValuePair<string, string>("", ""));
            UtilIU.LlenarValoresDesdeLista(comboUnmdItem, listUM);

            Documento = new GeneradorFacturaLib.Boleta();
        }
      
        

        /// <summary>
        /// Manejador del evento click del botón Cerrar
        /// </summary>
        /// <param name="sender">Control que lansa el evento</param>
        /// <param name="e">Evento</param>
        private void btnCerrar_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Ejecuta la validación del formulario.
        /// </summary>
        /// <returns>string con los mensajes de error producto de la validación</returns>
        private string ValidarForm()
        {
            // validar la seccion de encabezado(emisor, cliente, totales)
            string mensajesError = UtilIU.ValidarFormulario(this, new List<string> { "Encabezado", "Detalles" }, definicionesValidacion);
            return mensajesError;
        }

        /// <summary>
        /// Manejador del evento click del botón salvar.
        /// </summary>
        /// <param name="sender">Control que lansa el evento</param>
        /// <param name="e">Evento</param>
        private void btnSalvar_Click(object sender, EventArgs e)
        {
            // validar el formulario
            string mensajesError = ValidarForm();
            if (mensajesError != "")
            {
                MessageBox.Show(mensajesError, "Errores de Validación", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                #region IdDoc
                try
                {

                    long folio = 0;
                    long.TryParse(textFolio.Text, out folio);
                    var BolType = comboTipoDTE.SelectedValue.ToString();
                    Documento.SetEncabezadoIdDoc(
                       BolType,
                       folio,
                       dateFchEmis.Value);
                }
                catch (Exception ex)
                {
                    UtilIU.MostrarMensajeError(ex);
                    return;
                }
                #endregion

                #region Emisor
                try
                {
                    Documento.SetEncabezadoEmisor(
                        textRUTEmisor.Text,
                        textRznSoc.Text,
                        textGiroEmis.Text,
                        textDirOrigen.Text,
                        textCmnaOrigen.Text,
                        textCiudadOrigen.Text);
                }
                catch (Exception ex)
                {
                    UtilIU.MostrarMensajeError(ex);
                    return;
                }
                #endregion

                #region Receptor
                try
                {
                    Documento.SetEncabezadoReceptor(
                        textRUTRecep.Text,
                        textRznSocRecep.Text,
                        textDirRecep.Text,
                        textCmnaRecep.Text,
                        textCiudadRecep.Text);
                }
                catch (Exception ex)
                {
                    UtilIU.MostrarMensajeError(ex);
                    return;
                }
                #endregion

                #region Totales
                try
                {
                    long mntTotal = 0;
                    long.TryParse(textMntTotal.Text, out mntTotal);

                    long mntExe = 0;
                    long.TryParse(textMntExe.Text, out mntExe);

                    long mntNeto = 0;
                    long.TryParse(textMntNeto.Text, out mntNeto);

                    long IVA = 0;
                    long.TryParse(textIva.Text, out IVA);

                    Documento.SetEncabezadoTotales(mntTotal, mntExe, mntNeto, IVA);
                }
                catch (Exception ex)
                {
                    UtilIU.MostrarMensajeError(ex);
                    return;
                }
                #endregion

                var formSave = new FormSalvar(Documento, textIpDteBox.Text, textLlaveAutenticacion.Text);
                formSave.ShowDialog();
            }

        }

        /// <summary>
        /// Manejador del evento ValueChanged del componente textQtyItem(cantidad de la linea de detalle).
        /// Aquí se actualiza el valor del monto: cantidad * precio
        /// </summary>
        /// <param name="sender">Control que lansa el evento</param>
        /// <param name="e">Evento</param>
        private void textQtyItem_ValueChanged(object sender, EventArgs e)
        {
            textMontoItem.Value = textQtyItem.Value * textPrcItem.Value;
        }

        /// <summary>
        /// Manejador del evento Leave del componente textQtyItem(cantidad de la linea de detalle).
        /// Aquí se actualiza el valor del monto: cantidad * precio
        /// </summary>
        /// <param name="sender">Control que lansa el evento</param>
        /// <param name="e">Evento</param>
        private void textQtyItem_Leave(object sender, EventArgs e)
        {
            textMontoItem.Value = textQtyItem.Value * textPrcItem.Value;
        }

        /// <summary>
        /// Manejador del evento Load del formulario.
        /// </summary>
        /// <param name="sender">Control que lansa el evento</param>
        /// <param name="e">Evento</param>
        private void BolForm_Load(object sender, EventArgs e)
        {
            this.controlesEntrada = UtilIU.ObtenerControles(this, new List<Type> { typeof(ComboBox), typeof(TextBox) });
            var combos = this.controlesEntrada.FindAll(c => c.GetType() == typeof(ComboBox));
            foreach (var comb in combos)
            {
                (comb as ComboBox).SelectedIndex = 0;
            }

            this.Text = string.Format("Boleta - {0:dd-MM-yyyy HH:mm:ss}", DateTime.Now);

            UtilIU.ManejarFlujoEntidad(dataGridReferencia, definicionesValidacion);
            UtilIU.ManejarFlujoEntidad(dataGridDetalles, definicionesValidacion);
        }

        #region GridReferencia

        /// <summary>
        /// Llenar el grid de las referencias desde la lista de referencias del documento de Boleta.
        /// </summary>
        private void dataGridReferencia_Llenar()
        {
            foreach (var item in Documento.ListaReferencias)
            {
                dataGridReferencia.Rows.Add(
                    item.CodRef,
                    item.IsValidRazonRef ? item.RazonRef : "",
                    "");
            }
        }

        /// <summary>
        /// Método que se ejecuta producto de las acciones de adicionar o actualizar referencias.
        /// </summary>
        /// <param name="indice">indice del elemento a actualizar, si es menor que 0 entonces es adicionar sino actualizar</param>
        private void dataGridReferencia_AdicionarOActualizar(int indice)
        {
            if (indice < 0)
            {
                Documento.AdicionarReferencia(
                    textCodRef.Text,
                    textRazonRef.Text);
            }
            else
            {
                Documento.ActualizarReferenciaEnIndice(
                    textCodRef.Text,
                    textRazonRef.Text,
                    indice);

            }
        }

        /// <summary>
        /// Método que se ejecuta producto de la accion de editar, se produce al seleccionar una fila del grid.
        /// </summary>
        /// <param name="indice">indice del elemento a editar</param>
        private void dataGridReferencia_Editar(int indice)
        {
            var actual = Documento.ListaReferencias[indice];
            textCodRef.Text = actual.CodRef;
            textRazonRef.Text = actual.IsValidRazonRef ? actual.RazonRef : "";
            //textComentarioRef.Text = current.IsValidIdAdicOtr ? current.IdAdicOtr : "";
        }

        /// <summary>
        /// Método que se ejecuta producto de las accion de eliminar una referencia.
        /// </summary>
        /// <param name="indice">indice del elemento a eliminar</param>
        private void dataGridReferencia_Eliminar(int indice)
        {
            Documento.EliminarReferenciaEnIndice(indice);
        }

        /// <summary>
        /// Método que se ejecuta producto de las accion de limpiar el formulario de adicionar referencia.
        /// </summary>
        private void dataGridReferencia_Limpiar()
        {
            textCodRef.Clear();
            textRazonRef.Clear();
            textComentarioRef.Text = "";
        }

        #endregion

        #region GridDetalles

        /// <summary>
        /// Llenar el grid de los detalles desde la lista de detalles del documento de Boleta.
        /// </summary>
        private void dataGridDetalles_Llenar()
        {
            foreach (var detalle in Documento.ListaDetalles)
            {
                string descIndExe = "";
                string descUnmdItem = "";
                if (detalle.IsValidIndExe)
                {
                    var valueIndExe = UtilIU.ValorParaEnum(detalle.IndExe);
                    descIndExe = AsistenteEnum.ObtenerDescripcion(typeof(DocumentoBoleta.SiiDte.Enumerations.Detalle_IndExe), (int.Parse(valueIndExe) + 1).ToString());
                }
                if (detalle.IsValidUnmdItem && !string.IsNullOrEmpty(detalle.UnmdItem))
                {
                    descUnmdItem = AsistenteEnum.ListaUnidadMedida().Find(u => u.Key == detalle.UnmdItem).Value;
                }

                CdgItem actual = null;
                IEnumerator<CdgItem> enumer = detalle.CdgItem.GetEnumerator();
                while (enumer.MoveNext())
                {
                    actual = enumer.Current;
                    break;
                }

                dataGridDetalles.Rows.Add(
                    actual != null ? actual.VlrCodigo : "",
                    detalle.NmbItem,
                    detalle.IsValidDscItem ? detalle.DscItem : "",
                    descUnmdItem,
                    detalle.IsValidQtyItem ? detalle.QtyItem.ToString() : "",
                    detalle.IsValidPrcItem ? detalle.PrcItem.ToString() : "",
                    detalle.MontoItem,
                    descIndExe
                    );
            }
        }

        /// <summary>
        /// Método que se ejecuta producto de las acciones de adicionar o actualizar detalles.
        /// </summary>
        /// <param name="indice">indice del elemento a actualizar, si es menor que 0 entonces es adicionar sino actualizar</param>
        private void dataGridDetalles_AdicionarOActualizar(int indice)
        {
            ListaCdgItem.Clear();
            ListaCdgItem.Add(Documento.CrearDetalleCdgItem("INT", textCdgItem.Text));

            if (indice < 0)
            {
                Documento.AdicionarDetalle(
                    textNmbItem.Text,
                    long.Parse(textMontoItem.Text),
                    UtilIU.EstaMarcadoCheckbox(comboIndExe) ? comboIndExe.SelectedValue.ToString() : null,
                    textDscItem.Text,
                    textQtyItem.Value,
                    comboUnmdItem.SelectedValue.ToString(),
                    textPrcItem.Value,
                    ListaCdgItem
                );
            }
            else
            {
                Documento.ActualizarDetalleEnIndice(
                    textNmbItem.Text,
                    long.Parse(textMontoItem.Text),
                    UtilIU.EstaMarcadoCheckbox(comboIndExe) ? comboIndExe.SelectedValue.ToString() : null,
                    textDscItem.Text,
                    textQtyItem.Value,
                    comboUnmdItem.SelectedValue.ToString(),
                    textPrcItem.Value,
                    ListaCdgItem,
                    indice
                );
            }

        }

        /// <summary>
        /// Método que se ejecuta producto de la accion de editar, se produce al seleccionar una fila del grid.
        /// </summary>
        /// <param name="indice">indice del elemento a editar</param>
        private void dataGridDetalles_Editar(int indice)
        {
            var detalle = Documento.ListaDetalles[indice];
            if (detalle != null)
            {

                textNmbItem.Text = detalle.NmbItem;
                textMontoItem.Text = detalle.MontoItem.ToString();

                if (detalle.IsValidIndExe)
                {
                    var item = UtilIU.ValorParaEnum(detalle.IndExe);
                    if (item != null) comboIndExe.SelectedValue = (int.Parse(item) + 1).ToString();
                    chk_comboIndExe.Checked = true;
                }

                if (detalle.IsValidDscItem)
                {
                    textDscItem.Text = detalle.DscItem;
                }

                if (detalle.IsValidQtyItem)
                {
                    textQtyItem.Text = detalle.QtyItem.ToString();
                }

                if (detalle.IsValidUnmdItem)
                {
                    comboUnmdItem.SelectedValue = detalle.UnmdItem;
                }

                if (detalle.IsValidPrcItem)
                {
                    textPrcItem.Text = detalle.PrcItem.ToString();
                }

                IEnumerator<CdgItem> enumer = detalle.CdgItem.GetEnumerator();
                CdgItem actual = null;
                while (enumer.MoveNext())
                {
                    actual = enumer.Current;
                    break;
                }
                textCdgItem.Text = actual != null ? actual.VlrCodigo : "";
            }
        }

        /// <summary>
        /// Método que se ejecuta producto de las accion de eliminar un detalle.
        /// </summary>
        /// <param name="indice">indice del elemento a eliminar</param>
        private void dataGridDetalles_Eliminar(int index)
        {
            Documento.EliminarDetalleEnIndice(index);

            ListaCdgItem.Clear();
            //dataGridCdgItem_Fill();
        }

        /// <summary>
        /// Método que se ejecuta producto de las accion de limpiar el formulario de adicionar detalles.
        /// </summary>
        private void dataGridDetalles_Limpiar()
        {
            textCdgItem.Clear();
            textQtyItem.Value = new decimal(0.000000);
            textNmbItem.Clear();
            comboUnmdItem.SelectedValue = "";
            textMontoItem.Value = 0;
            textPrcItem.Value = new decimal(0.000000);
            textDscItem.Clear();
            comboIndExe.SelectedIndex = 0; chk_comboIndExe.Checked = false;
        }

        #endregion

        /// <summary>
        /// Método que permite activar o desabilitar los componentes asociados a un CheckBox que lansa el evento CheckedChanged.
        /// </summary>
        private void chk_CheckedChanged(object sender, EventArgs e)
        {
            var nombreControlAsociados = (sender as CheckBox).Name.Substring(4);
            var controles = this.Controls.Find(nombreControlAsociados, true);
            if (controles.Length > 0)
            {
                controles[0].Enabled = (sender as CheckBox).Checked;
            }
        }

        /// <summary>
        /// Manejador del evento CheckedChanged del CheckBox chk_comboIndExe.
        /// </summary>
        private void chk_comboIndExe_CheckedChanged(object sender, EventArgs e)
        {
            this.chk_CheckedChanged(sender, e);
        }

    }
}
