﻿using DocumentoDTE.SiiDte;
using System;
using System.Collections.Generic;
using System.Windows.Forms;
using GeneGeneradorFacturaLib;


namespace GenerarFactura
{
    /// <summary>
    /// Formulario para gestionar los DTE de tipo Exportaciones.
    /// </summary>
    public partial class ExpForm : Form
    {
        /// <summary>
        /// Lista para almacenar los controles de entrada.
        /// </summary>
        private List<Control> controlesEntrada;

        /// <summary>
        /// Lista para almacenar los códigos de la sección de detalles.
        /// </summary>
        public List<CdgItemB> ListaCdgItemB { set; get; }

        /// <summary>
        /// Definición de las diferentes reglas de validación para los campos de la interfaz de usuario en este formulario.
        /// </summary>
        public Dictionary<string, Validacion> definicionesValidacion = new Dictionary<string, Validacion>
        {
            {"textRUTEmisor", new Validacion {LabelCampo = "RUT del Emisor", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true}, {"rut", true} }}},
            {"textRznSoc", new Validacion {LabelCampo = "Razón social del Emisor", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textGiroEmis", new Validacion {LabelCampo = "Giro del Emisor", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textDirOrigen", new Validacion {LabelCampo = "Dirección del Emisor", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textCmnaOrigen", new Validacion {LabelCampo = "Comuna del Emisor", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textCiudadOrigen", new Validacion {LabelCampo = "Ciudad del Emisor", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textActeco", new Validacion {LabelCampo = "Acteco del Emisor", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textRUTRecep", new Validacion {LabelCampo = "RUT del Cliente", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true}, {"rut", true} }}},
            {"textRznSocRecep", new Validacion {LabelCampo = "Razón social del Cliente", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textGiroRecep", new Validacion {LabelCampo = "Giro del Cliente", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textDirRecep", new Validacion {LabelCampo = "Dirección del Cliente", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textCmnaRecep", new Validacion {LabelCampo = "Comuna del Cliente", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textCiudadRecep", new Validacion {LabelCampo = "Ciudad del Cliente", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},

            {"textFolio", new Validacion {LabelCampo = "Folio", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true}, {"num", true} }}},
            {"textMntNeto", new Validacion {LabelCampo = "Monto Neto", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"num", true} }}},
            {"textIva", new Validacion {LabelCampo = "IVA", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"num", true} }}},
            {"textMntExe", new Validacion {LabelCampo = "Monto Exento", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"num", true} }}},
            {"textMntTotal", new Validacion {LabelCampo = "Monto Total", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true}, {"num", true} }}},
            {"textIpDteBox", new Validacion {LabelCampo = "IP DTEBOX", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textLlaveAutenticacion", new Validacion {LabelCampo = "Llave de Autenticación", Seccion = "Encabezado", Reglas = new Dictionary<string, object> { {"req", true} }}},

            {"textNmbItem", new Validacion {LabelCampo = "Nombre", Seccion = "Detalle", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textQtyItem", new Validacion {LabelCampo = "Cantidad", Seccion = "Detalle", Reglas = new Dictionary<string, object> { {"req", true}, {"num", true}, {"min", new decimal(0.000001)} }}},
            {"textPrcItem", new Validacion {LabelCampo = "Precio U.", Seccion = "Detalle", Reglas = new Dictionary<string, object> { {"req", true}, {"num", true}, {"min", new decimal(0.000001)} }}},
            {"textMontoItem", new Validacion {LabelCampo = "Monto", Seccion = "Detalle", Reglas = new Dictionary<string, object> { {"req", true}, {"num", true} }}},

            {"comboTpoDocRef", new Validacion {LabelCampo = "Tipo de documento", Seccion = "Referencia", Reglas = new Dictionary<string, object> { {"req", true} }}},
            {"textFolioRef", new Validacion {LabelCampo = "Folio", Seccion = "Referencia", Reglas = new Dictionary<string, object> { {"req", true} }}},

            {"dataGridDetalles", new Validacion {LabelCampo = "Detalle", Seccion = "Detalles", Reglas = new Dictionary<string, object> { {"req", true} }}},
        };

        /// <summary>
        /// Exportación.
        /// </summary>
        public GeneradorFacturaLib.Exportaciones Exportaciones { set; get; }

        public ExpForm()
        {
            InitializeComponent();

            ListaCdgItemB = new List<CdgItemB>();

            UtilIU.LlenarValoresDesde<Enumerations.DetalleB_IndExe>(comboIndExe);

            var exp = new List<string> {"110","111","112"};
            var listTipoDoc = AsistenteEnum.ListaTipoDocumento().FindAll(i => exp.Contains(i.Key));
            UtilIU.LlenarValoresDesdeLista(comboTpoDocRef, listTipoDoc);
            UtilIU.LlenarValoresDesdeLista(comboTipoDTE, listTipoDoc);

            var listUM = AsistenteEnum.ListaUnidadMedida();
            listUM.Insert(0, new KeyValuePair<string, string>("", ""));
            UtilIU.LlenarValoresDesdeLista(comboUnmdItem, listUM);

            Exportaciones = new GeneradorFacturaLib.Exportaciones();
        }

        /// <summary>
        /// Manejador del evento click del botón Cerrar
        /// </summary>
        /// <param name="sender">Control que lansa el evento</param>
        /// <param name="e">Evento</param>
        private void btnCerrar_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Ejecuta la validación del formulario.
        /// </summary>
        /// <returns>string con los mensajes de error producto de la validación</returns>
        private string ValidarForm()
        {
            // validar la seccion de encabezado(emisor, cliente, totales)
            string mensajesError = UtilIU.ValidarFormulario(this, new List<string> { "Encabezado", "Detalles" }, definicionesValidacion);
            return mensajesError;
        }

        /// <summary>
        /// Manejador del evento click del botón salvar.
        /// </summary>
        /// <param name="sender">Control que lansa el evento</param>
        /// <param name="e">Evento</param>
        private void btnSalvar_Click(object sender, EventArgs e)
        {
            // validar el formulario
            string mensajesError = ValidarForm();
            if (mensajesError != "")
            {
                MessageBox.Show(mensajesError, "Errores de Validación", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                #region IdDoc
                try
                {

                    long folio = 0;
                    long.TryParse(textFolio.Text, out folio);
                    var ExpType = comboTipoDTE.SelectedValue.ToString();
                    Exportaciones.SetEncabezadoIdDoc(
                       ExpType,
                       folio,
                       dateFchEmis.Value);
                }
                catch (Exception ex)
                {
                    UtilIU.MostrarMensajeError(ex);
                    return;
                }
                #endregion

                #region Emisor
                try
                {
                    Exportaciones.SetEncabezadoEmisor(
                        textRUTEmisor.Text,
                        textRznSoc.Text,
                        textGiroEmis.Text,
                        textActeco.Text,
                        textDirOrigen.Text,
                        textCmnaOrigen.Text,
                        textCiudadOrigen.Text);
                }
                catch (Exception ex)
                {
                    UtilIU.MostrarMensajeError(ex);
                    return;
                }
                #endregion

                #region Receptor
                try
                {
                    Exportaciones.SetEncabezadoReceptor(
                        textRUTRecep.Text,
                        textRznSocRecep.Text,
                        textGiroRecep.Text,
                        textDirRecep.Text,
                        textCmnaRecep.Text,
                        textCiudadRecep.Text);
                }
                catch (Exception ex)
                {
                    UtilIU.MostrarMensajeError(ex);
                    return;
                }
                #endregion

                #region Totales
                try
                {
                    Exportaciones.SetEncabezadoTotales(textMntTotal.Value, textMntExe.Value);
                }
                catch (Exception ex)
                {
                    UtilIU.MostrarMensajeError(ex);
                    return;
                }
                #endregion

                var formSave = new FormSalvar(Exportaciones, textIpDteBox.Text, textLlaveAutenticacion.Text);
                formSave.ShowDialog();
            }

        }

        /// <summary>
        /// Manejador del evento ValueChanged del componente textQtyItem(cantidad de la linea de detalle).
        /// Aquí se actualiza el valor del monto: cantidad * precio
        /// </summary>
        /// <param name="sender">Control que lansa el evento</param>
        /// <param name="e">Evento</param>
        private void textQtyItem_ValueChanged(object sender, EventArgs e)
        {
            textMontoItem.Value = textQtyItem.Value * textPrcItem.Value;
        }

        /// <summary>
        /// Manejador del evento Leave del componente textQtyItem(cantidad de la linea de detalle).
        /// Aquí se actualiza el valor del monto: cantidad * precio
        /// </summary>
        /// <param name="sender">Control que lansa el evento</param>
        /// <param name="e">Evento</param>
        private void textQtyItem_Leave(object sender, EventArgs e)
        {
            textMontoItem.Value = textQtyItem.Value * textPrcItem.Value;
        }

        /// <summary>
        /// Manejador del evento Load del formulario.
        /// </summary>
        /// <param name="sender">Control que lansa el evento</param>
        /// <param name="e">Evento</param>
        private void ExpForm_Load(object sender, EventArgs e)
        {
            this.controlesEntrada = UtilIU.ObtenerControles(this, new List<Type> { typeof(ComboBox), typeof(TextBox) });
            var combos = this.controlesEntrada.FindAll(c => c.GetType() == typeof(ComboBox));
            foreach (var comb in combos)
            {
                (comb as ComboBox).SelectedIndex = 0;
            }

            this.Text = string.Format("Exportación - {0:dd-MM-yyyy HH:mm:ss}", DateTime.Now);

            UtilIU.ManejarFlujoEntidad(dataGridReferencia, definicionesValidacion);
            UtilIU.ManejarFlujoEntidad(dataGridDetalles, definicionesValidacion);
        }

        #region GridReferencia

        /// <summary>
        /// Llenar el grid de las referencias desde la lista de referencias del documento.
        /// </summary>
        private void dataGridReferencia_Llenar()
        {
            foreach (var item in Exportaciones.ListaReferencias)
            {
                string descTpoDocRef = AsistenteEnum.ListaTipoDocumento().Find(t => t.Key == item.TpoDocRef).Value;
                dataGridReferencia.Rows.Add(
                    descTpoDocRef,
                    item.FolioRef,
                    item.FchRef,
                    item.IsValidRazonRef ? item.RazonRef : "",
                    "");
            }
        }

        /// <summary>
        /// Método que se ejecuta producto de las acciones de adicionar o actualizar referencias.
        /// </summary>
        /// <param name="indice">indice del elemento a actualizar, si es menor que 0 entonces es adicionar sino actualizar</param>
        private void dataGridReferencia_AdicionarOActualizar(int indice)
        {
            if (indice < 0)
            {
                Exportaciones.AdicionarReferencia(
                    comboTpoDocRef.SelectedValue.ToString(),
                    textFolioRef.Text,
                    dateFchRef.Value.Date,
                    textRazonRef.Text);
            }
            else
            {
                Exportaciones.ActualizarReferenciaEnIndice(
                    comboTpoDocRef.SelectedValue.ToString(),
                    textFolioRef.Text,
                    dateFchRef.Value.Date,
                    textRazonRef.Text,
                    indice);

            }

        }

        /// <summary>
        /// Método que se ejecuta producto de la accion de editar, se produce al seleccionar una fila del grid.
        /// </summary>
        /// <param name="indice">indice del elemento a editar</param>
        private void dataGridReferencia_Editar(int indice)
        {
            var actual = Exportaciones.ListaReferencias[indice];
            comboTpoDocRef.SelectedValue = actual.TpoDocRef;
            textFolioRef.Text = actual.FolioRef;
            dateFchRef.Value = actual.FchRef.AsDateTime;
            textRazonRef.Text = actual.IsValidRazonRef ? actual.RazonRef : "";
            //textComentarioRef.Text = current.IsValidIdAdicOtr ? current.IdAdicOtr : "";
        }

        /// <summary>
        /// Método que se ejecuta producto de las accion de eliminar una referencia.
        /// </summary>
        /// <param name="indice">indice del elemento a eliminar</param>
        private void dataGridReferencia_Eliminar(int index)
        {
            Exportaciones.EliminarReferenciaEnIndice(index);
        }

        /// <summary>
        /// Método que se ejecuta producto de las accion de limpiar el formulario de adicionar referencia.
        /// </summary>
        private void dataGridReferencia_Limpiar()
        {
            comboTpoDocRef.SelectedIndex = 0;
            textFolioRef.Clear();
            dateFchRef.Value = DateTime.Now;
            textRazonRef.Clear();
            textComentarioRef.Text = "";
        }

        #endregion

        #region GridDetalles

        /// <summary>
        /// Llenar el grid de los detalles desde la lista de detalles del documento de Boleta.
        /// </summary>
        private void dataGridDetalles_Llenar()
        {
            foreach (var detalle in Exportaciones.ListaDetalles)
            {
                string descIndExe = "";
                string descUnmdItem = "";
                if (detalle.IsValidIndExe)
                {
                    var valueIndExe = UtilIU.ValorParaEnum(detalle.IndExe);
                    descIndExe = AsistenteEnum.ObtenerDescripcion(typeof(DocumentoDTE.SiiDte.Enumerations.DetalleB_IndExe), (int.Parse(valueIndExe) + 1).ToString());
                }
                if (detalle.IsValidUnmdItem && !string.IsNullOrEmpty(detalle.UnmdItem))
                {
                    descUnmdItem = AsistenteEnum.ListaUnidadMedida().Find(u => u.Key == detalle.UnmdItem).Value;
                }

                CdgItemB actual = null;
                IEnumerator<CdgItemB> enumer = detalle.CdgItem.GetEnumerator();
                while (enumer.MoveNext())
                {
                    actual = enumer.Current;
                    break;
                }

                dataGridDetalles.Rows.Add(
                    actual != null ? actual.VlrCodigo : "",
                    detalle.NmbItem,
                    detalle.IsValidDscItem ? detalle.DscItem : "",
                    descUnmdItem,
                    detalle.IsValidQtyItem ? detalle.QtyItem.ToString() : "",
                    detalle.IsValidPrcItem ? detalle.PrcItem.ToString() : "",
                    detalle.MontoItem,
                    descIndExe
                    );
            }
        }

        /// <summary>
        /// Método que se ejecuta producto de las acciones de adicionar o actualizar detalles.
        /// </summary>
        /// <param name="indice">indice del elemento a actualizar, si es menor que 0 entonces es adicionar sino actualizar</param>
        private void dataGridDetalles_AdicionarOActualizar(int index)
        {
            ListaCdgItemB.Clear();
            ListaCdgItemB.Add(Exportaciones.CrearDetalleCdgItem("INT", textCdgItem.Text));

            if (index < 0)
            {
                Exportaciones.AdicionarDetalle(
                    textNmbItem.Text,
                    textMontoItem.Value,
                    UtilIU.EstaMarcadoCheckbox(comboIndExe) ? comboIndExe.SelectedValue.ToString() : null,
                    textDscItem.Text,
                    textQtyItem.Value,
                    comboUnmdItem.SelectedValue.ToString(),
                    textPrcItem.Value,
                    ListaCdgItemB
                );
            }
            else
            {
                Exportaciones.ActualizarDetalleEnIndice(
                    textNmbItem.Text,
                    textMontoItem.Value,
                    UtilIU.EstaMarcadoCheckbox(comboIndExe) ? comboIndExe.SelectedValue.ToString() : null,
                    textDscItem.Text,
                    textQtyItem.Value,
                    comboUnmdItem.SelectedValue.ToString(),
                    textPrcItem.Value,
                    ListaCdgItemB,
                    index
                );
            }

        }

        /// <summary>
        /// Método que se ejecuta producto de la accion de editar, se produce al seleccionar una fila del grid.
        /// </summary>
        /// <param name="indice">indice del elemento a editar</param>
        private void dataGridDetalles_Editar(int indice)
        {
            var detalle = Exportaciones.ListaDetalles[indice];
            if (detalle != null)
            {

                textNmbItem.Text = detalle.NmbItem;
                textMontoItem.Text = detalle.MontoItem.ToString();

                if (detalle.IsValidIndExe)
                {
                    var item = UtilIU.ValorParaEnum(detalle.IndExe);
                    if (item != null) comboIndExe.SelectedValue = (int.Parse(item) + 1).ToString();
                    chk_comboIndExe.Checked = true;
                }

                if (detalle.IsValidDscItem)
                {
                    textDscItem.Text = detalle.DscItem;
                }

                if (detalle.IsValidQtyItem)
                {
                    textQtyItem.Text = detalle.QtyItem.ToString();
                }

                if (detalle.IsValidUnmdItem)
                {
                    comboUnmdItem.SelectedValue = detalle.UnmdItem;
                }

                if (detalle.IsValidPrcItem)
                {
                    textPrcItem.Text = detalle.PrcItem.ToString();
                }

                IEnumerator<CdgItemB> enumer = detalle.CdgItem.GetEnumerator();
                CdgItemB actual = null;
                while (enumer.MoveNext())
                {
                    actual = enumer.Current;
                    break;
                }
                textCdgItem.Text = actual != null ? actual.VlrCodigo : "";
            }
        }

        /// <summary>
        /// Método que se ejecuta producto de las accion de eliminar un detalle.
        /// </summary>
        /// <param name="indice">indice del elemento a eliminar</param>
        private void dataGridDetalles_Eliminar(int index)
        {
            Exportaciones.EliminarDetalleEnIndice(index);

            ListaCdgItemB.Clear();
            //dataGridCdgItem_Fill();
        }

        /// <summary>
        /// Método que se ejecuta producto de las accion de limpiar el formulario de adicionar detalle.
        /// </summary>
        private void dataGridDetalles_Limpiar()
        {
            textCdgItem.Clear();
            textQtyItem.Value = new decimal(0.000000);
            textNmbItem.Clear();
            comboUnmdItem.SelectedValue = "";
            textMontoItem.Value = 0;
            textPrcItem.Value = new decimal(0.000000);
            textDscItem.Clear();
            comboIndExe.SelectedIndex = 0; chk_comboIndExe.Checked = false;
        }

        #endregion

        /// <summary>
        /// Método que permite activar o desabilitar los componentes asociados a un CheckBox que lansa el evento CheckedChanged.
        /// </summary>
        private void chk_CheckedChanged(object sender, EventArgs e)
        {
            var associatedControlName = (sender as CheckBox).Name.Substring(4);
            var controls = this.Controls.Find(associatedControlName, true);
            if (controls.Length > 0)
            {
                controls[0].Enabled = (sender as CheckBox).Checked;
            }
        }

        /// <summary>
        /// Manejador del evento CheckedChanged del CheckBox chk_comboIndExe.
        /// </summary>
        private void chk_comboIndExe_CheckedChanged(object sender, EventArgs e)
        {
            this.chk_CheckedChanged(sender, e);
        }

        private void textLlaveAutorizacion_TextChanged(object sender, EventArgs e)
        {

        }

    }
}
