﻿using GeneradorFacturaLib;
using System;
using System.Windows.Forms;

namespace GenerarFactura
{
    /// <summary>
    /// Formulario para mostrar el XML generado a partir del objeto Factura y enviar la Factura al DTEBox.
    /// </summary>
    public partial class FormSalvar : Form
    {
        /// <summary>
        /// Factura a enviar al DTEBox.
        /// </summary>
        public IFactura Factura { set; get; }

        /// <summary>
        /// Dirección ip o nombre del servidor del DTEBox.
        /// </summary>
        public string IpDTEBox { get; set; }

        /// <summary>
        /// Llave de Autenticación para la conección al DTEBox.
        /// </summary>
        public string LlaveAutenticacion { get; set; }

        /// <summary>
        /// Constructor del Formulario.
        /// </summary>
        /// <param name="factura">Factura</param>
        /// <param name="ipDTEBox">Dirección ip o nombre del servidor del DTEBox.</param>
        public FormSalvar(IFactura factura, string ipDTEBox, string llaveAutenticacion)
        {
            InitializeComponent();
            Factura = factura;
            IpDTEBox = ipDTEBox;
            LlaveAutenticacion = llaveAutenticacion;
            textXMLFactura.Text = Factura.ObtenerXml();
        }

        /// <summary>
        /// Manejador del evento click del botón Enviar al DTEBox. Aqúi se envía la Factura al DTEBox.
        /// </summary>
        private void btnSalvar_Click(object sender, EventArgs e)
        {
            try
            {
                #region Llamada al servicio

                DTEBoxCliente.Ambiente ambiente = DTEBoxCliente.Ambiente.Homologacion;
                DateTime fechaResolucion = new DateTime(2012, 1, 1);
                int numeroResolucion = 0;
                DTEBoxCliente.TipoPdf417 tipoPdf417 = DTEBoxCliente.TipoPdf417.Fuente;

                string apiURL = "http://<DTEBox IP>/api/Core.svc/Core".Replace("<DTEBox IP>", IpDTEBox);
                string apiAuth = LlaveAutenticacion;

                DTEBoxCliente.Servicio servicio = new DTEBoxCliente.Servicio(apiURL, apiAuth);

                DTEBoxCliente.ResultadoEnvioDte resultadoEnvioDte = null;
                DTEBoxCliente.ResultadoEnvioBoleta resultadoEnvioBoleta = null;
                DTEBoxCliente.EstadoResultado resultado = DTEBoxCliente.EstadoResultado.Ok;
                if (Factura.GetType() == typeof(Exportaciones))
                {
                    resultadoEnvioDte = servicio.EnviarDocumento((Factura as Exportaciones).DTE, ambiente, fechaResolucion, numeroResolucion, tipoPdf417);
                    resultado = resultadoEnvioDte.ResultadoServicio.Estado;
                }
                else if (Factura.GetType() == typeof(Liquidacion))
                {
                    resultadoEnvioDte = servicio.EnviarDocumento((Factura as Liquidacion).DTE, ambiente, fechaResolucion, numeroResolucion, tipoPdf417);
                    resultado = resultadoEnvioDte.ResultadoServicio.Estado;
                }
                else if (Factura.GetType() == typeof(Documento))
                {
                    resultadoEnvioDte = servicio.EnviarDocumento((Factura as Documento).DTE, ambiente, fechaResolucion, numeroResolucion, tipoPdf417);
                    resultado = resultadoEnvioDte.ResultadoServicio.Estado;
                }
                else if (Factura.GetType() == typeof(Boleta))
                {
                    resultadoEnvioBoleta = servicio.EnviarDocumento((Factura as Boleta).DTE, ambiente, fechaResolucion, numeroResolucion, tipoPdf417);
                    resultado = resultadoEnvioBoleta.ResultadoServicio.Estado;
                }

                #endregion

                #region Procesar resultado

                //Si la respuesta es correcta
                if (resultado == DTEBoxCliente.EstadoResultado.Ok)
                {
                    //Usar el data que viene en el resultado de la llamada
                    //string ted = resultado.TED; //Xml que representa el elemento TED generado
                    //Si tipoPdf417 = DTEBoxCliente.TipoPdf417.Fuente viene el código, 
                    // si es DTEBoxCliente.TipoPdf417.Grafico, Entonces arreglo de bytes codigificado en base64, 
                    // usar método result.Pdf417ComoArregloBytes();
                    //string pdf417 = resultado.Pdf417;

                    //Código de usuario a partir de aquí
                    MessageBox.Show("Factura enviada correctamente al DTEBox", "Información", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else //Si la llamada no fue satisfactoria
                {
                    //Descripción del error, actuar acorde
                    string description = resultadoEnvioDte != null ? resultadoEnvioDte.ResultadoServicio.Descripcion : resultadoEnvioBoleta.ResultadoServicio.Descripcion;
                    MessageBox.Show(description, "Error enviado desde el DTEBox", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

                #endregion
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Error enviando la Factura al DTEBox", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Manejador del evento click del botón Cancelar. Cierra el Formulario.
        /// </summary>
        private void btnCancelar_Click(object sender, EventArgs e)
        {
            Close();
        }

    }
}
